function ExtractGaborParamList(infilename, numvec)

% the FFNN outputs a mat-file in 'testdir', in which the parameters from
% filter functions are stored. ExtractGaborParamList can be used to extract
% these parameters and store them in an ASCII file, which is needed for the
% gabor c program.
% The ASCII parameter list is stored in outdir (standard value is
% ./Gaborlists)

% Ex. ExtractGaborParamList('TIdigits_clean_ComplexGabor_s_and_t_and_ST_fdim23_abs_imag_real_80Feat_300I',61:80)
%
% IN:
% infilename : name of file containing optimal features/prototypes in Test.sSelectSetSort structure
%              these files are stored in ./testdir and 
% [num]: optional, vector of features nums to be used (higher number higher relevance)
%
% Michael Kleinschmidt, ICSI, Feb 2002

if (nargin ~= 2) & (nargin ~= 1)
  disp(' ')
  disp('Example: ExtractGaborParamList(filename (no extension), [21:80]) outputs the 60 most ');
  disp('         important filter functions');
  disp('Try one of these filenames as first parameter:')
  ls ./testdir/*Length.mat
  error('Incorrect number of parameters !')
end

outdir = './Gaborlists/';
% Load prototypes from file
eval(['load testdir/' infilename '.mat'])

% for older lists, type of envelope is not contained as one of the parameters
% in this case a gaussian envelope is assumed
if (isempty(findstr('hannenv', lower(infilename))) == 1) & (isempty(findstr('gausenv', lower(infilename))) == 1)
  disp('Filename doesn''t contain envelope type. Assuming Gaussian envelope.');
  matfiletype = 'oldstyle';
  infilename = [infilename '_gausEnv'];
end

% if no numvec is given, use all feature components
if ~exist('numvec')
  disp(['Using all feature components from 1 to ' num2str(length(Test.sSelectSetSort)) '.']);
  numvec = 1:length(Test.sSelectSetSort);
end

% define indexvector
if nargin==1
  vec=[1:length(Test.sSelectSetSort)];
else
  vec=numvec;
  if max(numvec)>length(Test.sSelectSetSort)
  length_Test = length(Test.sSelectSetSort)
	error('number in numvec to large!')
  end
end

fid=fopen([outdir '/' infilename '-' num2str(length(vec)) '.txt'],'w');

if findstr(lower(infilename),'gaus')
  disp('Gaussian envelope');
	% print first line
	fprintf(fid,['# Gabor parameter file (Gauss env). Units are 10ms and 1 channel, or 100Hz and 1cycl./channel\n']);
	fprintf(fid,'# \tf_0\tFmodF\tFmodT\tsigmaF\tsigmaT\tFext\tText\tmode\ttype\n');
	fprintf(fid,'# most important filter parameter have highest index (i.e. bottom of file)\n');
elseif findstr(lower(infilename),'hann')
	disp('Hanning envelope');
	% print first line
	fprintf(fid,['# Gabor parameter file (Hanning env). Units are 10ms and 1 channel, or 100Hz and 1cycl./channel\n']);
	fprintf(fid,'# \tf_0\tFmodF\tFmodT\tsigmaF\tsigmaT\tFextHanning\tTextHanning\tmode\ttype\n'); 	
	fprintf(fid,'# most important filter parameter have highest index (i.e. bottom of file)\n');
else
  disp(['Infilename: ' infilename]);
  error('Infilename does not contain type of envelope');
end

c=0;
for k=vec
	c=c+1;

	if Test.sSelectSetSort(k).prototype.FmodF==0
		Test.sSelectSetSort(k).prototype.type='temporal';
	elseif Test.sSelectSetSort(k).prototype.TmodF==0
		Test.sSelectSetSort(k).prototype.type='spectral';
	elseif Test.sSelectSetSort(k).prototype.TmodF>0
		Test.sSelectSetSort(k).prototype.type='STdown';
	else
		Test.sSelectSetSort(k).prototype.type='STup';
	end
	
	if findstr(lower(infilename),'gaus')
		
		fprintf(fid,'%i\t%i\t%.9e\t%.9e\t%.9e\t%.9e\t%i\t%i\t%s\t%s\n',c,Test.sSelectSetSort(k).prototype.Fcenter,...
		Test.sSelectSetSort(k).prototype.FmodF, Test.sSelectSetSort(k).prototype.TmodF,Test.sSelectSetSort(k).prototype.Fsigma,...
		Test.sSelectSetSort(k).prototype.Tsigma, Test.sSelectSetSort(k).prototype.Fext,Test.sSelectSetSort(k).prototype.Text,...
		char(Test.sSelectSetSort(k).prototype.mode),char(Test.sSelectSetSort(k).prototype.type));
  
	elseif findstr(lower(infilename),'hann')
	
    dummy = 0;        
    fprintf(fid,'%i\t%i\t%.9e\t%.9e\t%.9e\t%.9e\t%i\t%i\t%s\t%s\n',c,Test.sSelectSetSort(k).prototype.Fcenter,...
    Test.sSelectSetSort(k).prototype.FmodF,Test.sSelectSetSort(k).prototype.TmodF,dummy,...
    dummy,Test.sSelectSetSort(k).prototype.FextHanning,Test.sSelectSetSort(k).prototype.TextHanning,...
    char(Test.sSelectSetSort(k).prototype.mode),char(Test.sSelectSetSort(k).prototype.type));
       
	else
      error('Infilename does not contain type of envelope');
	end

end % of for

fclose(fid);

disp('Written feature list in ./Gaborlists');
disp([outdir '/' infilename '-' num2str(length(vec)) '.txt']);


%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   All rights reserved. 
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for purely research (excluding weapons 
%%   research) or educational purposes is granted, with the exception that 
%%   modified versions of the code may not be redistributed unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG.
%%
%%   Permission to use this software for weapons research or applications is 
%%   not granted.
%%
%%   No commercial exploitation of this software is permitted unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG. 
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%----------------------------------------------------------------------------- 
