function [melspec] = calc_melspec(vSignal, fs, plotflag)
%---------------------------------------------------------------------------------
% calculate melspectrum from input signal according to ETSI standard
% ETSI ES 201 108 V1.1.2 (2000-04)
%
% MELSPEC(SIGNAL, SAMPLINGFREQ. (HZ), PLOTFLAG)
%
% Feb. 2003 / Bernd Meyer / bernd.meyer@uni-oldenburg.de
%---------------------------------------------------------------------------------

% Set parameters
iVerboseFlag     = 0;
% script produces same output as the nokia frontend (delivered with Aurora2
% CDs when no pre-emphase is used
iUsePreemphase   = 0;

if fs == 8000
   % default values for 8 kHz input signals
   nfft = 256;                 % number of samples for fft
   WindowLength = 200;         % length of window function
   nW = hamming(WindowLength); % type of window function
   ShiftInt = 80;              % shift interval is 80 samples for 8 kHz
elseif fs == 11000
   % default values for 11 kHz input signals
   nfft = 256;                 % number of samples for fft
   WindowLength = 256;         % length of window function
   nW = hamming(WindowLength); % type of window function
   ShiftInt = 110;             % shift interval is 80 samples for 8 kHz
elseif fs == 16000
   % default values for 16 kHz input signals
   nfft = 512;                 % number of samples for fft
   WindowLength = 400;         % length of window function
   nW = hamming(WindowLength); % type of window function
   ShiftInt = 160;             % shift interval is 80 samples for 8 kHz
else   
   error(['sampling frequency of ' num2str(fs) ' Hz is not supported']);
end

%---------------------------------------------------------------------------------

if iVerboseFlag
   disp('Compute melspec with the following parameters: ');
   disp(['nfft = ' num2str(nfft) ', WindowLength = ' num2str(WindowLength) ...
      ' ShiftInt = ' num2str(ShiftInt) ' fs = ' num2str(fs)]);
end

nRows = size(vSignal,1);
nCols = size(vSignal,2);
if (nRows > 1) & (nCols > 1)
   error('only mono input supported')
elseif (nCols > 1)
   vInput = vInput';
end

% starting frequency of the filterbank
f_start = 64;     % Hz

% save original signal for sound 
vSignal_orig = vSignal;

% scale input signal
vSignal = vSignal * 2^15;

% offset compensation
vSignal_of = zeros(size(vSignal));
vSignal_of(1) = 0; % is this correct?
vSignal_of(2:length(vSignal)) = vSignal(2:length(vSignal)) - vSignal(2-1:length(vSignal)-1) + 0.999*vSignal_of(2-1:length(vSignal)-1);
vSignal = vSignal_of;
clear vSignal_of;

% framing: vSignalFrame(:,i) are all samples in ith time frame
i = 0;
for k = 1:ShiftInt:(length(vSignal)-WindowLength)
   i = i+1;
   vSignalFrame(:,i) = vSignal(k:k+WindowLength-1);
end
NumberOfFrames = i;

if iUsePreemphase
   for i=1:NumberOfFrames
       vSignalFrame(2:WindowLength,i) = vSignalFrame(2:WindowLength,i) - 0.97*vSignalFrame(2-1:WindowLength-1,i);
   end
end % of if

% windowing
for i=1:NumberOfFrames
   vSignalFrame(:,i) = vSignalFrame(:,i) .* hamming(WindowLength);
end

% fft
for i=1:NumberOfFrames
    spec_frame(:,i) = abs(fft(vSignalFrame(:,i),nfft));
end

% look only at freq bins 1..nfft/2+1
% bin(k) in etsi standard means bin(k+1) here!
bin = zeros(nfft/2+1,NumberOfFrames);

for i=1:NumberOfFrames
    bin(1:nfft/2+1,i) = spec_frame(1:nfft/2+1,i);
end
clear spec_frame;

% fc(i) is the center freq. of the ith mel channel
% fc(1) ~ 124 Hz, fc(23) ~ 3660 Hz
fc = eval_fc(fs,f_start);

% cbin(i) = center freq of the ith mel channel in terms of fft bin indices
% cbin(k) in ETSI paper means cbin(k+1) here!
cbin(25) = nfft/2;
cbin(1)  = round((f_start*nfft)/fs);
cbin(2:24) = round((fc*nfft)/fs);

% evaluate weighted sum of fft-bins in each band; use triangular, half-overlapping windows
[melspec] = fbank_eval(cbin,bin,fs,nfft,f_start,NumberOfFrames);

% plot compressed melspec
if plotflag
   openwindow(plotflag,[650 340 330 180],'Compressed Mel Spectrogram');   
   % h=findobj('Type','axes','Tag','MelspecWindow');
   % axes(h);
	imagesc(melspec);
	axis xy; % invert freq axis
	colormap jet;
	cb=colorbar;
	ylabel('Frequency bin');
	xlabel('Time frame');
end

%---------------------------------------------------------------------------------
function [fbank_comp] = fbank_eval(cbin,bin,fs,nfft,f_start,NumberOfFrames)
% evaluate compressed filterbank sum
for j=1:NumberOfFrames % index for time frames
	for k=1:23 % index for frequency bins
        
        i = cbin(k-1+1):cbin(k+1);
        sum1 = sum((i - cbin(k-1+1)+1)*bin(i+1,j) / (cbin(k+1) - cbin(k-1+1) +1));
	
        i = (cbin(k+1)+1):cbin(k+1+1);
        sum2 = sum(  [1 - ((i-cbin(k+1)) / (cbin(k+1+1)-cbin(k+1)+1))  ]*bin(i+1,j));
        
        sum3 = sum1 + sum2;
        
        % flooring
        if (sum3 < 2*10^(-22))
            % disp('flooring');
            sum3 = 2*10^(-22);
        end;
        
		  % non linear transformation      
        fbank_comp(k,j) = log(sum3);
	end

end

%--------------------------------------------------------------------------
function [fc] = eval_fc(fs,f_start)
% calculate center frequencies for all 23 bins
i=1:23;
fc = mel_inv(mel(f_start) + (mel(fs/2)-mel(f_start))*i / (23 + 1));

%--------------------------------------------------------------------------
function [y] = mel(x);
% calculate melfrequency y for input freq. x
y  = 2595*log10(1 + x./700);

%--------------------------------------------------------------------------
function [y] = mel_inv(x);
% calculate inverse melfrequency y for input freq. x
y = (10.^(x./2595)-1)*700;

%--------------------------------------------------------------------------
function []=openwindow(nummer,position,name) 
% Function for opening and positioning of figure windows 
% 
% Michael Kleinschmidt, AG MEDI, Uni Oldenburg   2.2.00

h=figure(nummer);
set(h,'MenuBar','no');
set(h,'NumberTitle','off');
set(h,'Name',name);
set(h,'Position',position);

%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   All rights reserved. 
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for purely research (excluding weapons 
%%   research) or educational purposes is granted, with the exception that 
%%   modified versions of the code may not be redistributed unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG.
%%
%%   Permission to use this software for weapons research or applications is 
%%   not granted.
%%
%%   No commercial exploitation of this software is permitted unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG. 
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%----------------------------------------------------------------------------- 