function z = gabor(x, t, Fsigma, Tsigma, FmodF, TmodF, envtype)
% generates 2D complex Gabor functions 
% optional: plot
%
% z: complex matrix with filter functions
% x: vector with grid points in frequency direction
% t: vector with grid points in time direction
% Fsigma, Tsigma: std deviation of the Gaussian
% FmodF, TmodF: frequency (not angular frequency!) of Euler's function
% envtype: type of envelope.  'hann' (for Hanning env) or 'gaus' (for Gaussian)
% Example: 
% func = gabor([-2:2],[-10:10],'nan','nan',0.12,0.07,'hann');
%
% bm: rewrote gabor.m so that the code resembles the c-version by David
% Gelbart
iVerboseFlag = 0;

Fcut = floor(length(x)/2); % extension in frequency direction
Tcut = floor(length(t)/2); % extension in time direction

nCols = 2*Tcut+1;

Fcut_start = x(1);
Fcut_stop  = x(length(x));
nRows      = length(x);

%--------------------------------------------------------------------------
% compute scale factor for hanning envelope
%--------------------------------------------------------------------------
sum_hanning   = 0;
scale_hanning = 0;
row = 1;
f_correct_count = 0;
func_gauss   = zeros(nRows,nCols);
func_hanning = zeros(nRows,nCols);
for f=[Fcut_start:Fcut_stop]   
    f_correct_count = f_correct_count + 1;
    col = 1;
    t_correct_count = 0;
    for t=[-Tcut:Tcut]
        t_correct_count = t_correct_count + 1;
        % /* Hanning envelope */
        hanning_w = 0.5 - 0.5*cos(2*pi*f_correct_count / (nRows+1));
        hanning_t = 0.5 - 0.5*cos(2*pi*t_correct_count / (nCols+1));
        envelope_hanning = hanning_w * hanning_t;
        sum_hanning = envelope_hanning + sum_hanning;        
        col=col+1;
    end
    
    row=row+1;
end
scale_hanning = 1/sum_hanning;

%------------------------------------------------------------------------
% compute envelope (hanning or gauss) & euler's function
%------------------------------------------------------------------------
f_correct_count = 0;
row = 1;

func_gauss   = zeros(nRows,nCols);
func_hanning = zeros(nRows,nCols);
for f=[Fcut_start:Fcut_stop]   
	f_correct_count = f_correct_count + 1;
	col = 1;
	t_correct_count = 0;
	for t=[-Tcut:Tcut]
		t_correct_count = t_correct_count + 1;
		%/* Complex Euler function */
		euler = exp(j*2*pi*FmodF*f) * exp(j*2*pi*TmodF*t);
		
	if findstr(envtype,'hann')
 		% /* Hanning envelope */
		hanning_w = 0.5 - 0.5*cos(2*pi*f_correct_count / (nRows+1));
		hanning_t = 0.5 - 0.5*cos(2*pi*t_correct_count / (nCols+1));
		envelope_hanning = scale_hanning * hanning_w * hanning_t;
	  % dummy(row,col) = envelope_hanning; % used to check correct scale factor
  	func_hanning(row,col) = envelope_hanning * euler;
    
  elseif findstr(envtype,'gaus')
			%/* Gaussian envelope */
			scale = 1/(2*pi*Fsigma*Tsigma);
			gauss_w = (-(f*f)) / (2*(Fsigma*Fsigma));
			gauss_t = (-(t*t)) / (2*(Tsigma*Tsigma));
			envelope_gauss = scale*exp(gauss_w+gauss_t);
			func_gauss(row,col)   = envelope_gauss   * euler;
		else
      	error('gabor.m: envtype must be either gaus or hann');
		end
	
  	col=col+1;
    
	end
	
	row=row+1;
  
end

if findstr(envtype,'hann')
  % s1 = sprintf('Using hanning envelope; sum(sum(envelope_hanning)): %0.5g\n',sum(sum(dummy))),
  if iVerboseFlag; disp('Using hanning envelope'); end
  z = func_hanning;
elseif findstr(envtype,'gaus')
  if iVerboseFlag; disp('Using gaussian envelope'); end
  z = func_gauss;
else
  error('envtype must be either gaus or hann');
end

if(0)
  figure
  imagesc(real(z))
  title('real(gabor))');
end

%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   All rights reserved. 
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for purely research (excluding weapons 
%%   research) or educational purposes is granted, with the exception that 
%%   modified versions of the code may not be redistributed unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG.
%%
%%   Permission to use this software for weapons research or applications is 
%%   not granted.
%%
%%   No commercial exploitation of this software is permitted unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG. 
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%----------------------------------------------------------------------------- 
