function prototype=proto(Param, prototype_number)

% Function for generating first set of feature prototypes from limits
%
% prototype_number indicates which prototype is to be replaced. If 0 then replace/generate all.
%
% Michael Kleinschmidt FFNN2.0, June 2001, Uni Oldenburg
% modified for complex Gabor functions: mk FFNN3.4c, Feb 2002, ICSI

if nargin<2
   prototype_number=0;
end

if findstr(Param.trialname, 'ComplexGabor')
   
   % 'ComplexGabor'       : complex Gabor filter (1D convolution,  C or Matlab function to be called)  
   % only one at a time
   
   prototype = ComplexGabormerk(Param.limits, Param.EnvType);
   
end

return

%--------------------------------------------------------------
function prototype=ComplexGabormerk(limits, EnvType);

% repeat until new feature is correct and different from 
% discarded feature

prototype.EnvType = EnvType;

% determine type
rand_num=rand(1);
if rand_num <= limits.Props(1)
  prototype.type = 'spectral';
elseif rand_num <= limits.Props(2)
  prototype.type = 'temporal';
elseif rand_num <= limits.Props(3)
  prototype.type = 'STup';
elseif rand_num <= limits.Props(4)
  prototype.type = 'STdown';
else
  error(['no type with ' num2str(rand_num)])
end 

% look at modulation frequencies
if findstr(prototype.type, 'STdown')
  logFmodF = Kzahl(log2(limits.FmodFmin), log2(limits.FmodFmax)); % equally dist in log2 domain
  prototype.FmodF=2^logFmodF;
  % Negative values result in downward ripples (upward in
  % nomenclature of Shamma's lab)
  logTmodF = Kzahl(log2(limits.TmodFmin), log2(limits.TmodFmax)); % equally dist in log2 domain
  prototype.TmodF=2^logTmodF;
elseif findstr(prototype.type, 'STup')
  logFmodF = Kzahl(log2(limits.FmodFmin), log2(limits.FmodFmax)); % equally dist in log2 domain
  prototype.FmodF=2^logFmodF;
  % Negative values result in downward ripples (upward in
  % nomenclature of Shamma's lab)
  logTmodF = Kzahl(log2(limits.TmodFmin), log2(limits.TmodFmax)); % equally dist in log2 domain
  prototype.TmodF=-1*2^logTmodF;
elseif findstr(prototype.type, 'spectral')
  logFmodF = Kzahl(log2(limits.FmodFmin), log2(limits.FmodFmax)); % equally dist in log2 domain
  prototype.FmodF=2^logFmodF;
  prototype.TmodF=0;
elseif findstr(prototype.type, 'temporal')
  logTmodF = Kzahl(log2(limits.TmodFmin), log2(limits.TmodFmax)); % equally dist in log2 domain
  prototype.TmodF=2^logTmodF;
  prototype.FmodF=0;
else
  error(['no modFs chosen with type ' prototype.type])
end

% eval. max. number of oscilliations
if (prototype.FmodF ~= 0)
    Fsigma_without_factor = floor(1.5*abs(1/prototype.FmodF/2));
    if Fsigma_without_factor ~= 0
      prototype.MaxTnumF = floor(limits.dim/2)/[floor(1.5*abs(1/prototype.FmodF/2))];
			if prototype.MaxTnumF < 0.4
        prototype.MaxTnumF = 0.4;
			elseif prototype.MaxTnumF > 3
        prototype.MaxTnumF = 3;
			end
		  % how many oscillations should fit into envelope?
		  % interval from -sigma to +sigma
      prototype.TnumF = Kzahl(0.4, prototype.MaxTnumF);
    end
end

if (prototype.TmodF ~= 0)
    Tsigma_without_factor = floor(1.5*abs(1/prototype.TmodF/2));
    if Tsigma_without_factor ~= 0
      prototype.MaxTnumT = floor(101/2)/[floor(1.5*abs(1/prototype.TmodF/2))];
		if prototype.MaxTnumT < 0.4
      prototype.MaxTnumT = 0.4;
		elseif prototype.MaxTnumT > 3
      prototype.MaxTnumT = 3;
		end

    prototype.TnumT = Kzahl(0.4,prototype.MaxTnumT);
  end
end

% set sigma (std of Gaussian envelope)
if prototype.TmodF
  prototype.Tsigma = abs(1/prototype.TmodF/2);
else 
  prototype.Tsigma = Kzahl(limits.Tsigmamin, limits.Tsigmamax);
end

if prototype.FmodF
  prototype.Fsigma = 1/prototype.FmodF/2;
else 
  prototype.Fsigma = Kzahl(limits.Fsigmamin, limits.Fsigmamax);
end

if prototype.FmodF
  prototype.Fsigma = prototype.TnumF/prototype.FmodF/2; 
else 
  prototype.Fsigma = Kzahl(limits.Fsigmamin, limits.Fsigmamax);
end

% set parameters: number of oscillations, extension of support
prototype.Fcutfac = Kzahl(limits.Fcutmin, limits.Fcutmax); 
prototype.Tcutfac = Kzahl(limits.Tcutmin, limits.Tcutmax);
prototype.Fext = floor(prototype.Fcutfac*prototype.Fsigma); % in channels
prototype.Text = floor(prototype.Tcutfac*prototype.Tsigma); % in 10ms frames

% set parameters for hanning envelope
if prototype.TmodF
   % set extension of hanning envelope according to TmodF
   prototype.HannTWidth = abs(prototype.TnumT/prototype.TmodF/2);
else
   % extension of hanning is variable with random value
   prototype.HannTWidth = Kzahl(limits.HannTWidthMin, limits.HannTWidthMax);
end

if prototype.FmodF
   prototype.HannFWidth = prototype.TnumF/prototype.FmodF/2;
else
   prototype.HannFWidth = Kzahl(limits.HannFWidthMin, limits.HannFWidthMax);
end

prototype.TextHanning = floor(prototype.Tcutfac*prototype.HannTWidth);
prototype.FextHanning = floor(prototype.Fcutfac*prototype.HannFWidth);

% set support
prototype.Fcutfac = Kzahl(limits.Fcutmin, limits.Fcutmax);
prototype.Tcutfac = Kzahl(limits.Tcutmin, limits.Tcutmax);
prototype.Fext = ceil(prototype.Fcutfac*prototype.Fsigma); % in channels
prototype.Text = ceil(prototype.Tcutfac*prototype.Tsigma); % in 10ms frames

% set mode
prototype.mode = limits.modes{Dzahl(1, length(limits.modes))};

% set center frequency
if findstr(prototype.EnvType,'hann')
	% for hanning envelope the center frequency Fcentervec depends from extension 
	% in freuqency direction
	Fcentervec = [floor(prototype.FextHanning/2)+1:limits.dim-floor(prototype.FextHanning/2)];
	prototype.Fcenter = Fcentervec(Dzahl(1, length(Fcentervec))); 
elseif findstr(prototype.EnvType,'gaus')
	Fcentervec = [floor(prototype.Fsigma)+1:limits.dim-floor(prototype.Fsigma)];
	prototype.Fcenter = Fcentervec(Dzahl(1, length(Fcentervec))); 
else
  error('Type of envelope must be either hann or gaus');
end

% response activation ('abs', 'greater_zero' or 'less_zero')
if strcmp(prototype.mode, 'mag')
  prototype.activation=limits.activation(1);
else
  prototype.activation=limits.activation(Dzahl(1, length(limits.activation)));
end

% build support
if findstr(prototype.EnvType,'hann')
    % build prototypes : support
    support=ones(limits.dim, 101);
    for k=1:limits.dim
      if ( k < (-prototype.FextHanning+prototype.Fcenter)) | ( k > (prototype.FextHanning+prototype.Fcenter) )
    	support(k, :)=0;
      end
    end
    for k=1:101
      if ( (k-51) < -prototype.TextHanning ) | ( (k-51) > prototype.TextHanning )
    	support(:, k)=0;
      end
    end
    prototype.support = support;
elseif findstr(prototype.EnvType,'gaus')
	% build prototypes : support
	support=ones(limits.dim, 101);
	for k=1:limits.dim
      if ( k < (-prototype.Fext+prototype.Fcenter)) | ( k > (prototype.Fext+prototype.Fcenter) )
		support(k, :)=0;
      end
	end
	for k=1:101
      if ( (k-51) < -prototype.Text ) | ( (k-51) > prototype.Text )
		support(:, k)=0;
      end
	end
	prototype.support = support;
else
    error('proto.m: prototype.EnvType must be hann or gaus');
end

% avoid using gauss-envelope-variables for hanning envelope and vice versa
if findstr(prototype.EnvType,'hann')
   prototype.Fext    = 'nan';
   prototype.Text    = 'nan';
   prototype.Fsigma  = 'nan';
   prototype.Tsigma  = 'nan';
   protoype.support  = 'nan';
   prototype.Fcutfac = 'nan';
   prototype.Tcutfac = 'nan';      
elseif findstr(prototype.EnvType,'gaus')
   prototype.FextHanning = 'nan';
   prototype.TextHanning = 'nan';
else
   error('prototype.EnvType must be either gaus or hann');
end

if findstr(prototype.EnvType,'hann')
	% build prototypes : Gabor in context (used for plots)
	prototype.Y = support.*gabor([1:limits.dim]-prototype.Fcenter, [-50:50], prototype.Fsigma, prototype.Tsigma, prototype.FmodF, prototype.TmodF,...
                 prototype.EnvType);
    
	% build prototypes : Gabor as time invers
	gabi = gabor([-prototype.FextHanning:prototype.FextHanning], [-prototype.TextHanning:prototype.TextHanning], prototype.Fsigma, prototype.Tsigma,...
           prototype.FmodF, -prototype.TmodF, prototype.EnvType);
elseif findstr(prototype.EnvType,'gaus')
	% build prototypes : Gabor in context
	prototype.Y = support.*gabor([1:limits.dim]-prototype.Fcenter, [-50:50], prototype.Fsigma, prototype.Tsigma, prototype.FmodF, prototype.TmodF,...
                  prototype.EnvType);
	% build prototypes : Gabor as time invers
	gabi = gabor([-prototype.Fext:prototype.Fext], [-prototype.Text:prototype.Text], prototype.Fsigma, prototype.Tsigma,...
           prototype.FmodF, -prototype.TmodF, prototype.EnvType);
end


if strcmp(prototype.mode, 'mag')
  prototype.Yinv = gabi;
elseif strcmp(prototype.mode, 'real')
  prototype.Yinv = real(gabi);
  bias = mean(mean(prototype.Yinv));
  prototype.Yinv = prototype.Yinv - bias;
elseif strcmp(prototype.mode, 'imag')
  prototype.Yinv = imag(gabi);
  bias = mean(mean(prototype.Yinv));
  prototype.Yinv = prototype.Yinv - bias;
else
  error('mode unknown')
end

if 0
  prototype
  
  figure(1111)
  imagesc(real(prototype.Y))
  axis xy
  colormap jet
  cb=colorbar;
  a=ylabel('Frequency');
  b=xlabel('Time');
  set(gca, 'FontSize', 14);
  set(a, 'FontSize', 14);
  set(b, 'FontSize', 14);
  title('New prototype in context')
  
  figure(1112)
  imagesc(real(prototype.Yinv))
  axis xy
  colormap jet
  cb=colorbar;
  a=ylabel('Frequency');
  b=xlabel('Time');
  set(gca, 'FontSize', 14);
  set(a, 'FontSize', 14);
  set(b, 'FontSize', 14);
  title('real template')
  
  figure(1113)
  imagesc(support)
  axis xy
  colormap jet
  cb=colorbar;
  a=ylabel('Frequency');
  b=xlabel('Time');
  set(gca, 'FontSize', 14);
  set(a, 'FontSize', 14);
  set(b, 'FontSize', 14);
  title('support')
  
  
  pause
end
  
  
return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5

function K=Kzahl(mini, maxi) %continuous random number

K = rand(1)*(maxi-mini)+mini;

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5

function D=Dzahl(mini, maxi) % discrete random number

D = floor(rand(1)*0.999*(maxi-mini+1))+mini;

return

%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   All rights reserved. 
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for purely research (excluding weapons 
%%   research) or educational purposes is granted, with the exception that 
%%   modified versions of the code may not be redistributed unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG.
%%
%%   Permission to use this software for weapons research or applications is 
%%   not granted.
%%
%%   No commercial exploitation of this software is permitted unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG. 
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%----------------------------------------------------------------------------- 