function run_ffnn(trialname,trainingflag,testflag,testlist)

% function run_ffnn(trialname,trainingflag,testflag,testlist)
% 
% set parameters and starts training function
% Gabor feature set is automatically optimized by using the FFNN
% (Feature-Finding neural network), a linear NN combined with an
% optimization rule ('substitution rule' after Gramss 1992) in combination
% with AdaBoost Algorithm (Freund & Schapire, 1996)
%
% trialname: string defining trial and parameters (define before first use!!!):
% '[OLLO|zifkom|TIdigits]_[noisy|clean]_ComplexGabor_[s_only|t_only|sandt_only|a_and_t_and_ST]_fdim23_[abs, real, imag]_[\d]Feat_[\d]Iter_[\d]BoostIt__[\d]Files_[\d]FmodTmin_[\d]FmodTmax'
% trainingflag: 1 indicates feature set optimization
% testflag: 1 indicates testing on unknown speech material
% testlist (optional): test conditions
%
% BoostIt: Number of Iterations for Boosting (1: ordinary FFNN)
% Feat: Number of Feature for the final set (Feat/BoostIt per BoostIteration subset)
% FmodTmin/max: denoting the (restricted) range of temporal modulation center freq in Hz 
%
% See HOWTO.TXT for examples

% display some examples if no input arguments are specified
if nargin == 0
  disp('No input arguments are specified')
  disp('Try something like');
  disp('run_ffnn(''OLLO_clean_ComplexGabor_s_and_t_and_ST_fdim23_abs_real_imag_1BoostIt_80Feat_1200Iter_hannEnv_10FLength'',1,1,{''clean''})');
  disp('run_ffnn(''OLLO_clean_ComplexGabor_sandt_only_fdim23_abs_real_1BoostIt_80Feat_1200Iter_hannEnv_10FLength'',1,1,{''clean''})'); 
  disp('run_ffnn(''OLLO_clean_ComplexGabor_s_and_t_and_ST_fdim23_abs_real_imag_8BoostIt_80Feat_1200Iter_gausEnv_10FLength'',1,1,{''clean''})'); 
  return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% get options from trialname %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Param=GetOptions(trialname);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% set variables             %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% visual       : BOOLEAN (1 for visual feedback during training, 0 no display)
Param.visual = 0; % 0;

% Set flag (1) for start with training (else load from file)
Param.TrainingFlag = trainingflag;

% Set flag (1) for testing classifier on (clean and noisy) test data 
Param.test_flag = testflag;

% Set types of test data (dir names)
Param.Testdatatype = testlist; %{'clean' 'ccitt.15dB' 'ccitt.10dB' 'ccitt.5dB'};

% set input dirs for testing
for t = 1:length(testlist)
    if findstr(Param.trialname,'TIMIT_')
        Param.CV.PrimeFeatDir(t).name =  [Param.datadir '/' char(testlist(t))];
    else
        Param.CV.PrimeFeatDir(t).name =  [Param.datadir '/test/' char(testlist(t))];
    end
end

% number of features to discard each iteration
Param.discardnum = 1;

% frequency of flooding
Param.Floodtime = 51;

% maximum allowed correlation between two features (across all examples)
Param.SimMax = 0.98; 

% -------------------------------------------------------------
% STARTING...

% Change dir to workdir
disp(Param.workdir)
eval(['cd ' Param.workdir])

% Initialize pseudo random process
rand('state', sum(100*clock));

% Start FFNN
fprintf('calling ffnn() with');
Param

if (Param.BoostIt == 1)
  % use ordinary FFNN
  Param.UseBoosting = 0;
else
  Param.UseBoosting = 1;
end

% disp(['UseBoosting has been set to ' num2str(Param.UseBoosting)]);

%%%%%%%%%%%%% Training %%%%%%%%%%%%%%%%%%%%%%%%

if Param.TrainingFlag

    % load input data
    Data.TRAIN.sPrimary=LoadIR(Param.TRAINPrimeFeatDir,Param,1,Param.alltrainnum);

    % Check if trialname already used
    if fileExists(['logdir/' Param.trialname '.mat']);
        %disp(['trialname ' Param.trialname ' already used!'])
        error('trialname already used! Use different trialname or do only testing!')
    end  
    
    Param.sumfeatures=0; % number of features already determined in the optimization 
    
    for j=1:Param.BoostIt
        
        % create or adjust distribution of weights
        if j==1
            % How many examples for training ?
            Param.P = length(Data.TRAIN.sPrimary);
            % Set D weights per sample for boosting (initialized to flat distribution)
            Param.Dvec = ones(1,Param.P)/Param.P;
            % matrix
            Param.Dmat = Param.Dvec;
        else % j>1
            % set new D weights; vCorrect = 1 for correctly classified, else = 0
            Param.Dvec = Param.beta(j-1)*Param.Dvec.*Sub(j-1).Test.TRAIN.res.vCorrect + Param.Dvec.*(1-Sub(j-1).Test.TRAIN.res.vCorrect);
            %normalize
            Param.Dvec = Param.Dvec / sum(sum(Param.Dvec));
            %Matrix
            Param.Dmat = [Param.Dmat; Param.Dvec];
        end
        Param.Mmin(j)=min(Param.Dvec);
        Param.Mmax(j)=max(Param.Dvec);
        Param.Mmean(j)=mean(Param.Dvec);
        fprintf('\nWeights relative to mean value between %4.2f and %4.2f \n',Param.Mmin/Param.Mmean,Param.Mmax/Param.Mmean);
        %plot
        if Param.visual
           MatrixPlotter(Param.Dmat,1001,'Dmat');
           figure(1004);semilogy(1:j,Param.Mmin./Param.Mmean,'b',1:j,Param.Mmax./Param.Mmean,'r');title('min(b) and max(r) weights relative to mean');
        end
        
        % Calculate number of features for Boost Iteration j
        Param.featurenum=ceil((Param.Allfeaturenum-Param.sumfeatures)/(Param.BoostIt-j+1));
        % Calculate total number of features that will be assigned after this
        % Boost Iteration j
        Param.sumfeatures=Param.sumfeatures+Param.featurenum;
        
        Sub(j).Test.TRAIN.res.Dscore = 0;
        while Sub(j).Test.TRAIN.res.Dscore <= 0.5 % repeat until at least 0.5 performance so that Boosting converges
            % Start FFNN with Param.featurenum features
            fprintf(['\n***** Start training sub set # ' num2str(j) '*****\n'])
            [Sub(j).Param,Sub(j).Test]=train_ffnn(Param,Data);
        end
            
        % Full set
        for k=1:Param.featurenum
            Test.sSelectSet(Param.sumfeatures-Param.featurenum+k) = Sub(j).Test.sSelectSet(k);
        end
        
        % Calculate error
        Param.eta(j) = 1 - Sub(j).Test.TRAIN.res.Dscore;
        Param.beta(j) = Param.eta(j) / ( 1 - Param.eta(j) );
        
        fprintf('\nError Eta: %6.4f\t Weight 1/Beta: %6.4f\n',Param.eta(j),1/Param.beta(j));
        
        %plot
        if Param.visual
           figure(1002);plot(1:j,Param.eta,'b',1:j,1-Param.eta,'k');title('Eta(b) and Dscore(k)');
           figure(1003);semilogy(1:j,Param.beta,'r',1:j,1./Param.beta,'c');title('Beta(r) and 1/Beta(c)');
        end
        
    end
    
    
    % retrain net and get result of full set
    Test.featurenum = length(Test.sSelectSet);
    Test=train_sort(Param,Data,Test);
    fprintf('\n Scoring for Complete Set\n')   
    fprintf('  TRAIN rate: %4.2f \t RMS: %6.4f\n',Test.TRAIN.rate*100,Test.TRAIN.RMS)

    % Calculate result based on boosting formula from subsets
    Test=GetYmodBoost(Test,Sub,Param);
    fprintf('\n Scoring for AdaBoost Combined Classifier\n')
    fprintf('  TRAIN rate: %4.2f \t RMS: %6.4f\n',Test.Boost.TRAIN.result.rate*100,Test.Boost.TRAIN.RMS)
    
    % save
    eval(['save testdir/' Param.trialname ' Param Test']);
    eval(['save logdir/' Param.trialname ' Param Test Sub']);
   
    
end

%%%%%%%%%%%%% Test %%%%%%%%%%%%%%%%%%%%%%%%

if Param.test_flag
    
    disp('Test FFNN')
    test_ffnn(Param);
    
end

disp('FINISHED !')

disp('Use plot_optimal_set_ordered to plot the set of modulation filters')
disp('Use ExtractGaborParamList to store filter parameters in ascii file (for feature extraction with gabor c-program)');

%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   All rights reserved. 
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for purely research (excluding weapons 
%%   research) or educational purposes is granted, with the exception that 
%%   modified versions of the code may not be redistributed unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG.
%%
%%   Permission to use this software for weapons research or applications is 
%%   not granted.
%%
%%   No commercial exploitation of this software is permitted unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG. 
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%----------------------------------------------------------------------------- 