function [Param,Test] = train_ffnn(Param,Data);

% Function for training of FFNN by substitution rule after Gramss (modified
% to accomodate big corpora and Gabor features)
% 
% Usage:
% function [Param,Test] = train_ffnn(Param,Data);
%
% Input:
% - Param : Structure of Parameters
% - Data  : Structure of input data
%
% Output:
% - Param : Structure of Parameters
% - Test  : Structure of test results
%
% Michael Kleinschmidt, AG MEDI, Uni Oldenburg   20.5.00
% modified by Michael Kleinschmidt, ICSI, Nov 2001
% and by Michael Kleinschmidt, ICSI, Jan 2002
% Universitt Oldenburg, May 2003

% changes
% mk, 20.11.99 (added dim, traindir and testdir as options)
% mk, 23.11.99 (added headeroffset as option)
% mk, 4.2.00 (added quickstart, window position, projection)
% mk, 8.2.00 (mean and std calculation)
% mk, 14.2.00 (noisedir as option)
% mk, 20.5.00 (ffnn -> train_ffnn)
% hg, 08.03.01 ( faster substitution method )
% mk, 4/01 (keeps best set of features on train/validation data)
% mk, 17.07.01 (revision leading to FFNN 2.0)
% mk, ICSI, Nov 2001 (adapt to Gabor and large corpora!!!)
% mk, ICSI, Jan 2002 (use fast algos for phoneme targets)
% mk, ICSI, Jan 2002 (word targets and integration over whole utterance,
% i.e. isolated words)
% mk, ICSI, Feb 2002 (now for continuous Parameters)
% mk, Oldenburg, May 2003 (includes AdaBoost)


if (nargin ~= 2)  
    error('Incorrect number of parameters ! Use "help" for correct usage !')
end

% TRAINING

disp('START TRAINING')


% derive limit values (contraints) from Param.trialname
Param=limits(Param);

%  open and position windows for visual training
if Param.visual
    OpenWindow(1,[720 180 300 200],'Recognition Rates') 
    OpenWindow(2,[700 450 200 130],'RMS Values')
    OpenWindow(111,[480 450 200 130],'Confusion Matrix')
    OpenWindow(10,[260 450 200 130],'Secondary Feature matrix X')
    OpenWindow(11,[40 450 200 130],'Target Matrix Y')
    OpenWindow(12,[40 270 200 130],'Result Matrix Y*') 
end	

% choose random start set of Gabor features and train net 
[Param,Data]=InitializeSet(Param,Data);

% initatilize global scoring variables
Param.Best.sSelectSet=Param.sSelectSet;
Param.Best.Iteration=0;
Param.Best.TRAIN.RMS=1000;
Param.Best.TRAIN.D_RMS=1000;
Param.Best.Dscore=0;

disp(' Start substitution process')
for i=1:Param.MaxIteration
    
    % how many features discarded this iteration ?
    if floor(i/Param.Floodtime) == i/Param.Floodtime
        discardnum=Param.discardnumFlood;
    else
        discardnum=Param.discardnum;
    end
    
    fprintf('\n Iteration # %i: ', i);
    
    % Calculate relevancies and RMS
    % disp('Param.Dvec:')
    [Param.total.relevances(i,1:Param.featurenum),error_without_ith_component(i,1:Param.featurenum+1)] = evaluate(Param.net,Data.TRAIN.mSecondaryRefined,Data.TRAIN.target,Param,Param.Dvec);
    Param.total.baseRMS(i)=error_without_ith_component(i,1);
    
    if Param.visual
        MatrixPlotter(Data.TRAIN.mSecondaryRefined,10,'X')
        MatrixPlotter(Param.net*Data.TRAIN.mSecondaryRefined,12,'result Y*')
    end
    
    [dummy,index_new] = sort(Param.total.relevances(i,1:Param.featurenum));
    index_new   = index_new(1:discardnum);
    disp(['discard feat ' num2str(index_new)])
    
    ok=0;
    while ~ok
        
        for counter=1:discardnum
            
            % find new Gabor function
            Param.sSelectSet(index_new(counter)).prototype=proto(Param);
            
            if findstr(Param.trialname,'_Diphone_')
                %Calculate TRAIN Gabor feature 
                tmp.mSecondaryRefinedTRAIN = FastCalcGaborMany(Data.TRAIN.sPrimary,Param.sSelectSet(index_new(counter)).prototype);
                % eliminate frames with class 0
                [tmp.dummy,Data.TRAIN.mSecondaryRefined(index_new(counter),:)]=EliminateZeroFrames(Data.TRAIN.vLabelsfull,tmp.mSecondaryRefinedTRAIN);
            else 
                %Calculate TRAIN Gabor feature 
                Data.TRAIN.mSecondaryRefined(index_new(counter),:) = FastCalcGaborMany(Data.TRAIN.sPrimary,Param.sSelectSet(index_new(counter)).prototype);
            end
            
        end % of for counter=1:discardnum
        
         % Check for double features
         ok=1;
         % C=corrcoef(Data.TRAIN.mSecondaryRefined');        
         % r = correl(x);
         x = Data.TRAIN.mSecondaryRefined';
         c = cov(x);
         d = sqrt(diag(c));
         % avoid division by zero warning
         if isempty(find(d==0))
            C = c./(d*d');
            % discard new feature if correlation coefficient with old 
            % feature is too high
            if max(max(abs(C-diag(diag(C))))) > Param.SimMax
               ok=0;
            end
         else % evaluation of corrcoeff would cause division by zero, 
            % so select new feature
            disp('Computation of correlation coefficient not possible; choosing new filter function. ')
            ok = 0;
         end % of if
        
    end  % of while ~ok
    
    % retrain net 
    [Param.net,tmp.Param.psi] = trainer(Data.TRAIN.mSecondaryRefined,Data.TRAIN.target,Param); 
    
    Param.total.D_RMS(i+1)=RMSerror(Param.net,Data.TRAIN.mSecondaryRefined,Data.TRAIN.target,Param.Dvec);
    Param.total.RMS(i+1)=RMSerror(Param.net,Data.TRAIN.mSecondaryRefined,Data.TRAIN.target);
    
    % Score new net
    [Param.total.result(i),tmp.Iteration(i).result.TRAINYmod] = classify(Param.net,Data.TRAIN.mSecondaryRefined,Data.TRAIN.target,Param.Dvec);
    Param.total.rate(i+1)=Param.total.result(i).rate;  
    Param.total.Dscore(i+1)=Param.total.result(i).Dscore; 
    
    fprintf('  TRAIN rate: %4.2f \t RMS: %6.4f \t D-RMS: %6.4f \t Dscore: %6.4f \n',Param.total.rate(i+1)*100,Param.total.RMS(i+1),Param.total.D_RMS(i+1),Param.total.Dscore(i+1))
    
    % new set better than previous favorite?
    if  (Param.total.D_RMS(i+1) < Param.Best.TRAIN.D_RMS)
        Param.Best.sSelectSet=Param.sSelectSet;
        Param.Best.Iteration=i;
        Param.Best.TRAIN.RMS=Param.total.RMS(i+1);
        Param.Best.TRAIN.D_RMS=Param.total.D_RMS(i+1);
        Param.Best.Dscore=Param.total.Dscore(i+1);
        Data.Best.TRAIN.mSecondaryRefined = Data.TRAIN.mSecondaryRefined;
        Data.Best.TRAIN.target = Data.TRAIN.target;
    end
    
    if Param.visual
        % Display confusion matrix
        MatrixPlotter(Param.total.result(i).matrix,111,'real / predicted')
    end
    
    if (Param.visual)	
        % plot recognition rates and relevance
        figure(1)
        plot([0:i],Param.total.rate*100,'b');
        xlabel('Iteration');
        ylabel('Recognition Rate [%]')
        title('Performance on Traindata')
        
        figure(2)
        plot([0:i],Param.total.D_RMS,'b');
        xlabel('Iteration');
        ylabel('D-RMS error');
        title('D-RMS on Traindata');
        
        figure(3)
        plot([0:i],Param.total.RMS,'b');
        xlabel('Iteration');
        ylabel('RMS error');
        title('RMS on Traindata');
        
        figure(4)
        plot([0:i],Param.total.Dscore,'b');
        xlabel('Iteration');
        ylabel('Dscore');
        title('Dscore on Traindata');
           
    end % if visual
    
end % of for i=1:Iteration 

% Test again and sort
Test=train_sort(Param,Data);

fprintf('\n Best set is %i\n',Param.Best.Iteration)
fprintf('  TRAIN rate: %4.2f \t RMS: %6.4f\t D-RMS: %6.4f \t Dscore: %6.4f \n',Test.TRAIN.rate*100,Test.TRAIN.RMS,Test.TRAIN.D_RMS,Test.TRAIN.res.Dscore)

return

%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   All rights reserved. 
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for purely research (excluding weapons 
%%   research) or educational purposes is granted, with the exception that 
%%   modified versions of the code may not be redistributed unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG.
%%
%%   Permission to use this software for weapons research or applications is 
%%   not granted.
%%
%%   No commercial exploitation of this software is permitted unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG. 
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%----------------------------------------------------------------------------- 