function vSecondary = FastCalcGaborFeat(mIR, prototype, display)

% Function for extracting secondary Gabor features from mIR
% (spectro-temporal representation) ONE Gabor function and only one
% feature!!! 
% combines CalcGaborFeat and RefineGaborFeat into fast routine for
% calculation of only one feature
% 
% call by:
% vRefinedFeature = ReCalcGaborFeat(mIR, prototype, RefineInfo, display)
%
% IN/OUT:
% vRefinedFeature : secondary Feature vector
%
% mIR : primary feature matrix
% prototype: parameters for one Gabor function
% display: flag for visual
%
% Michael Kleinschmidt, ICSI, Jan 2002
% Universitaet Oldenburg, Feb 2003

if nargin ~= 3
   error('Incorrect number of parameters !')
end

if findstr(prototype.EnvType,'gaus')
  Fext = prototype.Fext;
  Text = prototype.Text;
elseif findstr(prototype.EnvType,'hann')
  Fext = prototype.FextHanning;
  Text = prototype.TextHanning;
end

% pad top and bottom if necessary
y1 = prototype.Fcenter - Fext;
y2 = prototype.Fcenter + Fext;
mIRtemp = mIR(max(1,y1):min(y2,size(mIR,1)),:);

if y1 < 1
  vBottom = mIRtemp(1,:);
  mBottom = ones(1-y1,1)*vBottom;
  mIRtemp = [mBottom; mIRtemp];
end

if y2 > size(mIR,1)
  vTop = mIR(size(mIR,1),:);
  mTop = ones(y2-size(mIR,1),1)*vTop;
  mIRtemp = [mIRtemp; mTop];
end

% pad left and right
xpad = Text;

% padd input matrix to avoid onset and offset at edges
% left and right
vFirst = mIRtemp(:,1);
vLast = mIRtemp(:,size(mIRtemp,2));
mFirst = vFirst*ones(1,xpad);
mLast = vLast*ones(1,xpad);
mIRtemp = [mFirst mIRtemp mLast];

% do a number of 1D convolutions with reversed template
%
% convolution over time
for k=1:size(prototype.Yinv,1)
  r(k,:)=conv(mIRtemp(k,:),prototype.Yinv(k,:));
end

% cut off temporal padding and convolution extension
s=r(:,2*xpad+1:size(r,2)-(2*xpad));

% summation over frequencies
vSecondary=sum(s,1);

if strcmp(prototype.mode,'mag')
  vSecondary=abs(vSecondary);  
elseif strcmp(prototype.mode,'real')
  % do nothing
elseif strcmp(prototype.mode,'imag')
  % do nothing
else
  error(['prototype.Type ' prototype.type ' unknown!'])
end

if length(vSecondary)~=size(mIR,2)
  disp(['Length of vSecondary: ' num2str(length(vSecondary))]);
  disp(['Size of ST representation: ' num2str(size(mIRtemp))]);
  disp(['Size of filter function: '   num2str(size(prototype.Yinv))]);
  error('feature vector too short !!!')
end

if display
  subplot(331)
  imagesc(mIR)
  title('IR')
  mIRSize=size(mIR)
  subplot(332)
  imagesc(real(prototype.Yinv))
  YinvSize=size(prototype.Yinv)
  title('real(Yinv)')
  subplot(333)
  imagesc(real(prototype.Y))
  title('real(Y)')
  subplot(334)
  imagesc(mIRtemp)
  IRpadSize=size(mIRtemp)
  title('padded IR')
  subplot(335)
  imagesc(real(r))
  rSize=size(r)
  title('real(correlate with extension)')
  subplot(336)
  imagesc(real(s))
  sSize=size(s)
  title('real(correlate without extension)')
  subplot(337)
  imagesc(vSecondary)
  vSecSize=size(vSecondary)
  title('sum over frequencies');
  % pause
end

return
%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   All rights reserved. 
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for purely research (excluding weapons 
%%   research) or educational purposes is granted, with the exception that 
%%   modified versions of the code may not be redistributed unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG.
%%
%%   Permission to use this software for weapons research or applications is 
%%   not granted.
%%
%%   No commercial exploitation of this software is permitted unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG. 
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%----------------------------------------------------------------------------- 
