function RMS = RMSerror(W, X, Y, Dvec) 

% calculates the total euclidiann error
% Syntax: RMS=RMSerror(W,X,Y,Dvec)  calculates D-weighted RMS
%         RMS=RMSerror(W,X,Y)       calculates RMS
%         RMS=RMSerror(Y,Ymod) calculates RMS from Ymod (which was computed earlier)
%
% Input: 
% -- W is the (M*N) weight matrix if the neural net 
%
% -- Y is the (M*P) target matrix
%
% -- X is a matrix containing secondary feature vectors
%     where X(number of feature, number of ST representation) corresponds to features
%	
% -- D is a diagonal P*P matrix, that contains boosting weights 
%     per example
%	
% Output:
% -- E is the total euclidian error
%
% Michael Kleinschmidt, AG MEDI, Uni Oldenburg   19.6.99

if nargin ==4 
    
    % create matrix from Dvec
    D=diag(Dvec);
    
    % determine the difference between current and desired value (weighted with D)
    mDiff = Y - W * X; % result is a M*P matrix
    
    % calculate the total euclidian error
    mE = (mDiff' * mDiff); % result is a P*P matrix
    E = diag(mE)' * diag(D); % normed sum over errors for all examples with linear weight D
    
    RMS = sqrt(E / size(Y, 1)); % normed to all classes (not to examples)
    
elseif nargin ==3
    
    % calculate difference between current and desired value
    mDiff = Y - W * X;
    
    % calculate the total euclidian error
    E = trace(mDiff * mDiff');
    
    RMS = sqrt(E / size(Y, 1)/ size(Y, 2)); % normed to all classes and examples
    
    % Tino Gramss' dissertation, page 30, equation 13
    
elseif nargin ==2
    
    Y=W;
    Ymod=X;
    
    % calculate difference between current and desired value
    mDiff = Y - Ymod;
    
    % calculate the total euclidian error
    E = trace(mDiff * mDiff');
    
    RMS = sqrt(E / size(Y, 1)/ size(Y, 2)); % normiert auf alle Klassen und Beispiele
    
else
    error('Wrong number of arguments when calling RMSerror!')    
end

    % Tino Gramss' dissertation, page 30, equation 13

return
%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   All rights reserved. 
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for purely research (excluding weapons 
%%   research) or educational purposes is granted, with the exception that 
%%   modified versions of the code may not be redistributed unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG.
%%
%%   Permission to use this software for weapons research or applications is 
%%   not granted.
%%
%%   No commercial exploitation of this software is permitted unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG. 
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%----------------------------------------------------------------------------- 