function [relevances, err_without_ith_feat] = evaluate(W, X, Y, Param, D) 

% calculates the relevances of each feature with repeated adapation.
% problems with training (inverted matrix)!, so Xnew is filled with
% ones, not with zeros
%
% Syntax: [vRelevances, vErrorWithoutI] = evaluate(W, X, Y, D)
%
% Input: 
% -- W is the (M*N) weight matrix if the neural net 
%
% -- Y is the (M*P) target matrix
%
% -- X is a matrix containing secondary feature vectors
%     where X(number of feature, number of ST representation) corresponds to features
%	
% -- D is a diagonal P*P matrix, that contains boosting weights 
%     per example
%	
% Output:
% -- vRelevances is a N-dimensional vector containing the relevances of features
% -- vErrorWithoutI is a N+1-dimensional vector containing the quadratic error
%    without the ith feature. vErrorWithoutI(N+1) contains the error with all features
%
% Michael Kleinschmidt, AG MEDI, Uni Oldenburg   19.6.99

% what size are N (number of features) and P (number of patterns) ?
dim = size(X);
N   = dim(1);
P   = dim(2);

% calculate the total euclidian error with all features
if nargin == 5
    err_all_feat = RMSerror(W, X, Y, D);
else
    err_all_feat = RMSerror(W, X, Y);
end

% iteration over all features
for i = 1:N
    
    % set feature vector i equal to one
    Xnew = X;
    Xnew(i, :) = ones(size(X(i, :)));
    if nargin == 5
        % new NN training
        [Wnew, pseudo] = trainer(Xnew, Y, Param);
        % calculate the total euclidian error without feature i
        err_without_ith_feat(i) = RMSerror(Wnew, Xnew, Y, D);
    else
        % retrain neural net
        [Wnew, pseudo] = trainer(Xnew, Y, Param);
        % calculate the total euclidian error without feature i
        err_without_ith_feat(i) = RMSerror(Wnew, Xnew, Y);
    end        
      
    % calculate relevance of feature i
    relevances(i) = err_without_ith_feat(i) - err_all_feat;
    
end % N (features)

err_without_ith_feat(N+1) = err_all_feat;

% Tino Gramss' dissertation, page 38, equation 19

return

%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   All rights reserved. 
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for purely research (excluding weapons 
%%   research) or educational purposes is granted, with the exception that 
%%   modified versions of the code may not be redistributed unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG.
%%
%%   Permission to use this software for weapons research or applications is 
%%   not granted.
%%
%%   No commercial exploitation of this software is permitted unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG. 
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%----------------------------------------------------------------------------- 
