function [W, Psi] = trainer(X, Y, Param) 
% training of weight matrix using the pseudo inverse matrix
%
% Syntax: [net, pseudo] = trainer(features, manyIR.label, Param)
%
% Input: 
% -- W is the (M*N) weight matrix if the neural net 
%
% -- Y is the (M*P) target matrix
%
% -- X is a matrix containing secondary feature vectors
%     where X(number of feature, number of ST representation) corresponds to features
%	
% -- D is a diagonal P*P matrix, that contains boosting weights 
%     for each example
%
% -- Dvec is the diagonal of D with length P
%	
% Output:
% -- W ist the weight matrix of the classifying neural net with
%     Y  =   W  * X 
% (M*P)  = (M*N) * (N*P)
% -- Psi is the pseudo inverse of X(N*P, features*patterns)
%
% M = number of classes
% N = number of features
% P = number of patterns% 
%
% Michael Kleinschmidt, AG MEDI, Uni Oldenburg   19.6.99

% how many features ?
temp = size(X);
N = temp(1);

% how many patterns ?
P = temp(2);

%if nargin==3 % no boosting, old pseudoinverse method
if ~Param.UseBoosting
    if (P >= N) 
        XX = X * X';
        if rank(XX) == size(X,1)
           Psi = inv(XX);
           W = Y * X' * Psi;
        else
           disp(['Matrix can not be inverted.']);
           save
           error('training stopped');
        end
    else
        X_X = X' * X;
        if rank(X_X) == size(X,1)
           Psi = inv(X_X);
           W = Y * Psi * X';
        else
           disp(['Matrix can not be inverted.']);
           save 
           error('training stopped');
        end
        
    end
    
else % use P for boosting

   % generate matrix from Param.Dvec
    D=diag(Param.Dvec);
    
    % the dependency between error and D should be linearly
    D=sqrt(D);
    
    if (P >= N) 
        Psi = inv(X*D*D*X');
        W = Y * D*D * X' * Psi;
    else 
        Psi = inv(D * X' * X * D);
        W = Y * D*D * Psi * X';
    end
    
end

% Tino Gramss' dissertation, page 31, equation 14

return

%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   All rights reserved. 
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for purely research (excluding weapons 
%%   research) or educational purposes is granted, with the exception that 
%%   modified versions of the code may not be redistributed unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG.
%%
%%   Permission to use this software for weapons research or applications is 
%%   not granted.
%%
%%   No commercial exploitation of this software is permitted unless permission 
%%   has been obtained separately from UNIVERSITAET OLDENBURG. 
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%----------------------------------------------------------------------------- 